# -*- coding: utf-8 -*-

# tvdownloadingconfig.py
# This file is part of Qarte-3
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2012-2016 Vincent Vande Vyvre
# Licence: GPL3
# Home page: https://launchpad.net/qarte
#
# Download config dialog

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import (Qt, QCoreApplication, )
from PyQt5.QtGui import (QFont, )
from PyQt5.QtWidgets import (QDialog, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QLabel, QLineEdit,
                             QCheckBox, QRadioButton, QButtonGroup, QFrame, 
                             QTableWidget, QTableWidgetItem, QAbstractItemView,
                             QPushButton, )


class TvDownloadingConfig(QDialog):
    def __init__(self, driver, parent=None):
        super().__init__(parent)
        self.driver = driver
        self.resize(650, 600)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(4, 2, 4, 4)
        self.verticalLayout = QVBoxLayout()
        self.hl = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                       QSizePolicy.Minimum)
        self.hl.addItem(spacer)
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("DejaVu Sans")
        font.setPointSize(12)
        font.setWeight(75)
        font.setBold(True)
        self.title_lbl.setFont(font)
        self.title_lbl.setAlignment(Qt.AlignCenter)
        self.hl.addWidget(self.title_lbl)
        spacer1 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl.addItem(spacer1)
        self.verticalLayout.addLayout(self.hl)
        self.hl_2 = QHBoxLayout()
        self.filename_lbl = QLabel(self)
        font = QFont()
        font.setFamily("DejaVu Sans")
        font.setWeight(75)
        font.setBold(True)
        self.filename_lbl.setFont(font)
        self.hl_2.addWidget(self.filename_lbl)
        spacer2 = QSpacerItem(40, 17, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl_2.addItem(spacer2)
        self.verticalLayout.addLayout(self.hl_2)
        self.hl_3 = QHBoxLayout()
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.hl_3.addItem(spacer3)
        self.filename_led = QLineEdit(self)
        self.hl_3.addWidget(self.filename_led)
        self.verticalLayout.addLayout(self.hl_3)
        self.hl_4 = QHBoxLayout()
        spacer4 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.hl_4.addItem(spacer4)
        self.info_name_lbl = QLabel(self)
        self.info_name_lbl.setText("")
        self.info_name_lbl.hide()
        self.hl_4.addWidget(self.info_name_lbl)
        self.verticalLayout.addLayout(self.hl_4)
        self.hl_5 = QHBoxLayout()
        self.gridLayout_1 = QGridLayout()
        spacer5 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.gridLayout_1.addItem(spacer5, 0, 0, 1, 1)
        self.replace_space_chb = QCheckBox(self)
        self.gridLayout_1.addWidget(self.replace_space_chb, 0, 1, 1, 1)
        self.underscore_rdo = QRadioButton(self)
        self.underscore_rdo.setChecked(True)
        self.underscore_rdo.setEnabled(False)
        self.buttonGroup = QButtonGroup(self)
        self.buttonGroup.addButton(self.underscore_rdo)
        self.gridLayout_1.addWidget(self.underscore_rdo, 0, 2, 1, 1)
        spacer6 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.gridLayout_1.addItem(spacer6, 1, 0, 1, 1)
        spacer7 = QSpacerItem(118, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.gridLayout_1.addItem(spacer7, 1, 1, 1, 1)
        self.hyphen_rdo = QRadioButton(self)
        self.hyphen_rdo.setEnabled(False)
        self.buttonGroup.addButton(self.hyphen_rdo)
        self.gridLayout_1.addWidget(self.hyphen_rdo, 1, 2, 1, 1)
        self.date_chb = QCheckBox(self)
        self.gridLayout_1.addWidget(self.date_chb, 2, 1, 1, 1)
        self.hl_5.addLayout(self.gridLayout_1)
        spacer8 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl_5.addItem(spacer8)
        self.verticalLayout.addLayout(self.hl_5)

        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.hl_6 = QHBoxLayout()
        self.versions_lbl = QLabel(self)
        self.hl_6.addWidget(self.versions_lbl)
        spacer9 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl_6.addItem(spacer9)
        self.verticalLayout.addLayout(self.hl_6)
        self.versions_tbl = VersionTable(self)
        self.verticalLayout.addWidget(self.versions_tbl)
        self.hl_7 = QHBoxLayout()
        self.choice_lbl = QLabel(self)
        self.choice_lbl.setFont(font)
        self.hl_7.addWidget(self.choice_lbl)
        spacer10 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl_7.addItem(spacer10)
        self.verticalLayout.addLayout(self.hl_7)
        self.hl_8 = QHBoxLayout()
        self.choice_chb = QCheckBox(self)
        self.hl_8.addWidget(self.choice_chb)
        spacer11 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hl_8.addItem(spacer11)
        self.close_btn = QPushButton(self)
        self.hl_8.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.hl_8)  

        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.setWindowTitle(_("Downloading Parameters"))
        self.title_lbl.setText(_("Downloading configuration"))
        self.filename_lbl.setText(_("File name:"))
        self.replace_space_chb.setText(_("Replace space with:"))
        self.underscore_rdo.setText(_("Underscore"))
        self.hyphen_rdo.setText(_("Hyphen"))
        self.date_chb.setText(_("Add date"))
        self.versions_lbl.setText(_("Available versions:"))
        self.choice_chb.setText(_("Always use this quality / version"))
        self.close_btn.setText(_("Close"))
        self.replace_space_chb.stateChanged.connect(self.on_replace_checked)
        self.underscore_rdo.toggled.connect(self.set_separator)
        self.date_chb.stateChanged.connect(self.on_date_checked)
        self.choice_chb.stateChanged.connect(self.on_default_choice_changed)
        self.close_btn.clicked.connect(self.closeEvent)

    def configure(self, versions, video, quality, lang, sort=True, date=True):
        lgg.info('Configure dialog with lang: %s' %lang)
        self.versions = versions
        self.video = video
        self.preferred = quality
        self.title = video.fname
        self.filename_led.setText(self.title)
        self.sep = " "
        if date:
            self.date_chb.hide()

        if not sort:
            self.display_one_list(versions, quality)
            return

        if lang == 'fr':
            availables = self.sort_french_versions(versions)
            categories = "Français", "Allemand", "Autres"

        else:
            availables = self.sort_german_versions(versions)
            categories = "Deutsch", "Französisch", "Anderen"

        row = 0
        for i in range(3):
            self.versions_tbl.insertRow(row)
            item = QTableWidgetItem()
            item.setText(categories[i])
            self.versions_tbl.setItem(row, 0, item)
            row += 1
            for k in availables[i]:
                val = versions[k]
                mime = val["mediaType"]
                self.versions_tbl.insertRow(row)
                self.versions_tbl.setRowHeight(row, 20)
                item = QTableWidgetItem()
                item.setText(mime)
                self.versions_tbl.setItem(row, 0, item)
                item = QTableWidgetItem()
                item.setText(str(val["width"]))
                self.versions_tbl.setItem(row, 1, item)
                item.setTextAlignment(Qt.AlignRight)
                item = QTableWidgetItem()
                item.setText(str(val["height"]))
                self.versions_tbl.setItem(row, 2, item)
                item.setTextAlignment(Qt.AlignRight)
                item = QTableWidgetItem()
                item.setText(str(val["bitrate"]))
                self.versions_tbl.setItem(row, 3, item)
                item.setTextAlignment(Qt.AlignRight)
                item = QTableWidgetItem()
                item.setText("    %s" % val["versionLibelle"])
                self.versions_tbl.setItem(row, 4, item)
                row += 1

        try:
            idx = availables[0].index(self.preferred) + 1
            self.current_choice = self.preferred
        except (IndexError, ValueError):
            idx = 0
            self.current_choice = availables[0]

        item = self.versions_tbl.item(idx, 0)
        self.availables = [None]
        self.availables.extend(availables[0])
        self.availables.append(None)
        self.availables.extend(availables[1])
        if availables[2]:
            self.availables.append(None)
            self.availables.extend(availables[2])
        self.versions_tbl.setCurrentItem(item)

    def display_one_list(self, availables, quality):
        row = 0
        self.availables = []
        for k in availables:
            self.availables.append(k)
            val = availables[k]
            mime = val["mediaType"]
            self.versions_tbl.insertRow(row)
            self.versions_tbl.setRowHeight(row, 20)
            item = QTableWidgetItem()
            item.setText(mime)
            self.versions_tbl.setItem(row, 0, item)
            item = QTableWidgetItem()
            item.setText(str(val["width"]))
            self.versions_tbl.setItem(row, 1, item)
            item.setTextAlignment(Qt.AlignRight)
            item = QTableWidgetItem()
            item.setText(str(val["height"]))
            self.versions_tbl.setItem(row, 2, item)
            item.setTextAlignment(Qt.AlignRight)
            item = QTableWidgetItem()
            item.setText(str(val["bitrate"]))
            self.versions_tbl.setItem(row, 3, item)
            item.setTextAlignment(Qt.AlignRight)
            item = QTableWidgetItem()
            item.setText("    %s" % val["versionLibelle"])
            self.versions_tbl.setItem(row, 4, item)
            row += 1

        try:
            idx = self.availables.index(quality)
            self.current_choice = quality
        except:
            lgg.info('%s not in available qualities' % quality)
            idx = 0
        item = self.versions_tbl.item(idx, 0)
        self.versions_tbl.setCurrentItem(item)


    def sort_french_versions(self, streams):
        """Sort the streams in the order french, german, other.

        args:
        streams -- dict(stream items)
        """
        firsts = []
        seconds = []
        others = []
        keys = sorted(streams.keys())
        for key, value in streams.items():
            val = value["versionLibelle"].lower()
            if 'français' in val or 'muet' in val:
                firsts.append(key)
            elif 'allemand' in val:
                seconds.append(key)
            else:
                others.append(key)

        return [firsts, seconds, others]

    def sort_german_versions(self, streams):
        """Sort the streams in the order german, french, other.

        args:
        streams -- dict(stream items)
        """
        firsts = []
        seconds = []
        others = []
        keys = sorted(streams.keys())
        for key, value in streams.items():
            val = value["versionLibelle"].lower()
            if 'deutsch' in val or 'muet' in val:
                firsts.append(key)
            elif 'französisch' in val:
                seconds.append(key)
            else:
                others.append(key)

        return [firsts, seconds, others]
            
    def on_selection_changed(self, new, old):
        if new is None:
            self.reverse_selection(old)
            return

        idx = new.row()
        if self.availables[idx] is None:
            self.reverse_selection(old)
            return

        choice = self.availables[idx]
        data = self.versions[choice]
        self.choice_lbl.setText("%s %s x %s bitrate: %s vers: %s" %(
                                data["mediaType"], data["width"],
                                data["height"], data["bitrate"],
                                data["versionLibelle"]))
        self.current_choice = choice
        self.set_checkBox()

    def set_checkBox(self):
        valid = self.current_choice.endswith('_1')
        self.choice_chb.setEnabled(valid)
        self.choice_chb.setChecked(self.current_choice == self.preferred)

    def reverse_selection(self, item):
        self.versions_tbl.blockSignals(True)
        self.versions_tbl.clearSelection()
        QCoreApplication.processEvents()
        self.versions_tbl.setCurrentItem(item)
        self.versions_tbl.blockSignals(False)

    def on_default_choice_changed(self, state):
        if state:
            self.preferred = self.current_choice

    def on_replace_checked(self, state):
        self.underscore_rdo.setEnabled(state)
        self.hyphen_rdo.setEnabled(state)
        if state:
            self.set_separator()

        else:
            self.filename_led.setText(self.title)
            self.sep = " "

    def on_date_checked(self, state):
        suffix = ""
        if state:
            suffix = "-" + self.video.date.strip("'\" ")
        self.filename_led.blockSignals(True)
        self.filename_led.setText(self.video.title+suffix)
        self.filename_led.blockSignals(False)

    def set_separator(self):
        sep = "" + ("_" if self.underscore_rdo.isChecked() else "-")
        self.filename_led.blockSignals(True)
        self.filename_led.setText(self.filename_led.text()
                                    .replace(self.sep, sep))
        self.sep = sep
        self.filename_led.blockSignals(False)

    def closeEvent(self, event=None):
        self.video.fname = self.filename_led.text()
        self.video.quality = self.current_choice
        self.driver.set_preferred_quality(self.preferred)
        self.accept()


class VersionTable(QTableWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.dialog = parent
        self.setShowGrid(False)
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setAlternatingRowColors(True)
        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().hide()
        self.setColumnCount(5)
        self.setColumnWidth(0, 65)
        self.setColumnWidth(1, 60)
        self.setColumnWidth(2, 60)
        self.setColumnWidth(3, 80)
        labels = ['Type', 'Width', 'Height', 'Bitrate', 'Version']
        self.setHorizontalHeaderLabels(labels)
        self.currentItemChanged.connect(parent.on_selection_changed)







